from itertools import groupby
from django.forms.models import (
    ModelChoiceIterator, ModelChoiceField, ModelMultipleChoiceField
)

class Grouped(object):
    def __init__(self, queryset, group_by_field,
                 group_label=None, value_label=None, *args, **kwargs):
        """
        ``group_by_field`` is the name of a field on the model to use as
                           an optgroup.
        ``group_label`` is a function to return a label for each optgroup.
        """
        super(Grouped, self).__init__(queryset, *args, **kwargs)
        self.group_by_field = group_by_field
        if group_label is None:
            self.group_label = lambda group: group
        else:
            self.group_label = group_label

        if value_label is not None:
            self.label_from_instance = lambda obj: getattr(obj, value_label)

    def _get_choices(self):
        if hasattr(self, '_choices'):
            return self._choices
        return GroupedModelChoiceIterator(self)

class GroupedModelChoiceIterator(ModelChoiceIterator):
    def __iter__(self):
        if self.field.empty_label is not None:
            yield ("", self.field.empty_label)
        queryset = self.queryset.all()
        if not queryset._prefetch_related_lookups:
            queryset = queryset.iterator()
        for group, choices in groupby(self.queryset.all(),
                    key=lambda row: getattr(row, self.field.group_by_field)):
            if self.field.group_label(group):
                yield (
                    self.field.group_label(group),
                    [self.choice(ch) for ch in choices]
                )

class GroupedModelChoiceField(Grouped, ModelChoiceField):
    choices = property(Grouped._get_choices, ModelChoiceField._set_choices)

class GroupedModelMultiChoiceField(Grouped, ModelMultipleChoiceField):
    choices = property(Grouped._get_choices, ModelMultipleChoiceField._set_choices)



import types
from datetime import datetime, date

from django.utils import formats
from django.utils.timezone import localtime
from django.conf import settings

def formatDate(dt):
    if dt is None:
        return 'n.d.'
    return formats.date_format(dt, "SHORT_DATE_FORMAT")

def formatDateTime(dt):
    if dt is None:
        return 'n.d.'
    if settings.USE_TZ:
        dt = localtime(dt)
    return formats.date_format(dt, "SHORT_DATETIME_FORMAT")

def fieldlabel(obj, key):
    try:
        return obj._meta.get_field(key).verbose_name.strip().capitalize() # strip serve per forzare l'esecuzione di ugettext_lazy
    except:
        try:
            return getattr(obj, '%s_label' % key)()
        except:
            return key.capitalize().replace('_', ' ')

def fieldvalue(obj, key):
    try:
        return getattr(obj, 'get_%s_display' % key)()
    except:
        try:
            ret = getattr(obj, key)
        except:
            ret = None
        if type(ret) == types.MethodType:
            ret = ret()
        if ret is None:
            ret = ''
        elif isinstance(ret, datetime):
            ret = formatDateTime(ret)
        elif isinstance(ret, date):
            ret = formatDate(ret)
        return ret



from crispy_forms.helper import FormHelper

def form_horizontal_helper(request):
    helper = FormHelper()
    helper.form_tag = False
    helper.form_class = 'form-horizontal'
    helper.label_class = 'col-lg-4'
    helper.field_class = 'col-lg-8'
    helper.include_media = False
    return {
        'form_horizontal_helper': helper
    }

def table_inline_formset(request):
    helper = FormHelper()
    helper.form_tag = False
    helper.template = 'lazycrud/table_inline_formset.html'
    return {
        'table_inline_formset': helper
    }
